// EnvCell.cpp: implementation of the EnvCell class.
//
//////////////////////////////////////////////////////////////////////

#include "EnvCell.h"
#include "Utilities.h"

#include <scew.h>
#include <stdio.h>

// ============================================================================
EnvCell::EnvCell()
{
  intCount = 0;
  values = NULL;

  intPos = -1;
  x = y = 0.0;
}

// ============================================================================
EnvCell::EnvCell(int ct, BYTE * val)
{
  intCount = ct;
  values = val;

  intPos = -1;
  x = y = 0.0;
}

// ============================================================================
EnvCell::EnvCell(int ct, BYTE * val, int pos, double px, double py)
{
  intCount = ct;
  values = val;

  intPos = pos;
  x = px;
  y = py;
}

// ============================================================================
EnvCell::~EnvCell()
{
}

// ============================================================================
void EnvCell::setSize(int newSize)
{
  intCount = newSize;
}

// ============================================================================
int EnvCell::size()
{
  return intCount;
}

// ============================================================================
void EnvCell::setValues(BYTE * newValues)
{
  values = newValues;
}

// ============================================================================
BYTE * EnvCell::getValues()
{
  return values;
}

// ============================================================================
bool EnvCell::operator==(EnvCell oCell)
{
  int i;

  // if the sizes are different than they are not equal
  if (intCount != oCell.intCount)
    return false;

  // check if the values in both cells are identical
  i = 0;
  while ((values[i] == oCell.values[i]) && (i < intCount))
    i++;

  // if (i == intCount) then all values are equal
  // otherwise, the values at index [i] are the first difference found
  return (i == intCount);
}

// ============================================================================
bool EnvCell::operator!=(EnvCell oCell)
{
  return !(operator==(oCell));
}

// ============================================================================
EnvCell * EnvCell::fromXML(char * strXML)
{
  EnvCell * cell;
  int iResult;

    scew_tree* oTree = NULL;
    scew_parser* oParser = NULL;

  oParser = scew_parser_create();

    // Loads an XML file
    iResult = scew_parser_load_buffer(oParser, (const char *) strXML, strlen(strXML));

  if (iResult)
  {
    cell = fromXML(scew_tree_root(oTree));

    // Remember to free tree (scew_parser_free does not free it)
    // Frees the SCEW parser
    scew_tree_free(oTree);
    scew_parser_free(oParser);

    return cell;
  }

  else
  {
    // Remember to free tree (scew_parser_free does not free it)
    // Frees the SCEW parser
    scew_tree_free(oTree);
    scew_parser_free(oParser);

    throw GarpException(22, "Could not load node from XML text.");
	return NULL;
  }
}

// ============================================================================
EnvCell * EnvCell::fromXML(scew_element const * pXMLCellNode)
{
  int ct, i;
  char * strToken, * strCellValues, strAux[1024];
  BYTE * values;
  EnvCell * cell;

  scew_element * pXMLNode;

  //addref should have been called by the load method
  if (pXMLCellNode == NULL)
    throw GarpException(24, "Could not read EnvCell. XML node is empty.");

  //pXMLNode = pXMLCellNode->selectSingleNode("//EnvCell");
  pXMLNode = (scew_element *) pXMLCellNode;

  if (pXMLNode)
  {
    // first get number of values
    strToken = strCellValues = NULL;

    //get cell values
    strCellValues = (char* ) scew_element_contents(pXMLNode);

    ct = 0;
    strcpy(strAux, strCellValues);
    strToken = strtok(strAux, " \t\n\r");
    while (strToken != NULL)
    {
      strToken = strtok(NULL, " \t\n\r");
      ct++;
    }

    // create cell and parse values
    values = new BYTE[ct];
    cell = new EnvCell(ct, values);

    i = 0;
    strcpy(strAux, strCellValues);
    strToken = strtok(strAux, " \t\n\r");
    while (strToken != NULL)
    {
      values[i] = atoi(strToken);
      strToken = strtok(NULL, " \t\n\r");
      i++;
    }

    if (i != ct)
      throw GarpException(28, "Internal error: EnvCell::fromXML(): Error parsing node value.");

    return cell;
  }

  else
    throw GarpException(25, "XML source does not contain an EnvCell node.");

  return NULL;
}

// ============================================================================
char * EnvCell::toXML()
{
  const int iStringSize = 1024;

  char * strXML = new char[iStringSize];
  char strAux[256];

  sprintf(strXML, "<EnvCell Pos=\"%d\" X=\"%f\" Y=\"%f\">", intPos, x, y);

  for (int i = 0; i < intCount; i++)
  {
    sprintf(strAux, "%d ", values[i]);
    strcat(strXML, strAux);
  }

  strcat(strXML, "</EnvCell>\n");

  if (strlen(strXML) > iStringSize)
    throw GarpException(82, "String size exceeded in EnvCell::toXML()");

  return strXML;
}

// ============================================================================
// ============================================================================
