/**
 *    '$RCSfile: DBDataTypeResolver.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/08/13 00:58:45 $'
 *   '$Revision: 1.3 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.dataquery;

import java.util.Hashtable;

import org.apache.xpath.XPathAPI;

import org.ecoinformatics.seek.datasource.UnresolvableTypeException;
import org.ecoinformatics.util.Config;
import org.ecoinformatics.util.DebugPrinter;

import org.w3c.dom.NodeList;
import org.w3c.dom.Node;

/**
 * This class will resolve a data type in HSQL for a given data type in
 * Attribute object
 * @author Jing Tao
 *
 */

public class DBDataTypeResolver
{
  Hashtable mappingDBandAttribute = new Hashtable();
  Hashtable mappingJavaAndAttribute = new Hashtable();
  DebugPrinter dbg  = DebugPrinter.getInstance();
  private static final String DEFAULTTYPE = "LONGVARCHAR";
  private static final String JAVADEFAULT = "String";

  /**
   * Constructor  get two mapping from lib/config.xml file.
   * Mapping is hash table, key is attribute data type and value is db data type
   * Another mapping is : key - attribute data type, value - java data type
   */
  public DBDataTypeResolver()
  {
    try
    {
      NodeList nl = new Config().getDBdataTypeMappings();
      for(int i=0; i<nl.getLength(); i++)
      {
        Node n = nl.item(i);
        Node dbDataTypeNode = XPathAPI.selectSingleNode(n, "./DBdataType");
        Node attributeDataTypeNode =
                        XPathAPI.selectSingleNode(n, "./attributeDataType");
        Node javaDataTypeNode = XPathAPI.selectSingleNode(n, "./javaDataType");
        if( dbDataTypeNode == null ||  attributeDataTypeNode == null ||
            javaDataTypeNode == null)
        {
          throw new RuntimeException("Expecting a DBdataType or " +
                                     "AttributeDataType node " +
                                     "javaDataType node " +
                                     "child of mapping in " +
                                      "config.xml.");
        }

        String dbDataType = dbDataTypeNode.getFirstChild().getNodeValue();
        String attributeDataType =
                     attributeDataTypeNode.getFirstChild().getNodeValue();
        String javaDataType = javaDataTypeNode.getFirstChild().getNodeValue();
        dbg.print("The db data type " + dbDataType + " and attribute datatype" +
                  attributeDataType + " will be put into mapping hash", 2 );
        mappingDBandAttribute.put(attributeDataType, dbDataType);
        dbg.print("attribute datatype" + attributeDataType +
                  " and java type " + javaDataType +
                  " will be put into mapping hash", 2 );
        mappingJavaAndAttribute.put(attributeDataType, javaDataType);
      }//for

    }
    catch(javax.xml.transform.TransformerException te)
    {
      throw new RuntimeException("Error executing xpath expression in " +
        "DBDataTypeResolver.");
    }
    catch(Exception e)
    {
      throw new RuntimeException("Exception in DBDataTypeResolver.");
    }

  }

  /**
   * Method to find mapping db data type to a given attribute data type
   * @param attributeType String
   * @throws UnresolvableTypeException
   * @return String
   */
  public String resolveDBType(String attributeType)
                                             throws UnresolvableTypeException
  {
    String dbType = null;
    if (attributeType == null || attributeType.trim().equals(""))
    {
       dbType = DEFAULTTYPE;
    }
    else
    {
       dbType = (String)mappingDBandAttribute.get(attributeType);
    }

    if (dbType == null)
    {
      throw new UnresolvableTypeException("Unable to resolve db data type " +
                                          "for attribute data type " +
                                          attributeType);
    }
    return dbType;
  }

  public String resolveJavaType(String attributeType) throws UnresolvableTypeException
  {
    String javaType = null;
    if (attributeType == null || attributeType.trim().equals(""))
    {
       javaType = JAVADEFAULT;
    }
    else
    {
       javaType = (String)mappingJavaAndAttribute.get(attributeType);
    }

    if (javaType == null)
    {
      throw new UnresolvableTypeException("Unable to resolve java data type " +
                                          "for attribute data type " +
                                          attributeType);
    }
    return javaType;

  }



}
