/**
 *    '$RCSfile: Eml200DataSource.java,v $'
 *
 *     '$Author: higgins $'
 *       '$Date: 2004/07/26 21:56:50 $'
 *   '$Revision: 1.21 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.datasource.eml.eml2;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;
import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.ecoinformatics.ecogrid.client.EcogridQueryClient;
import org.ecoinformatics.seek.datasource.Attribute;
import org.ecoinformatics.seek.datasource.DataType;
import org.ecoinformatics.seek.datasource.TableEntity;
import org.ecoinformatics.seek.ecogrid.ResultRecord;
import org.ecoinformatics.seek.ecogrid.ResultTreeRoot;
import org.ecoinformatics.util.DelimitedReader;
import org.xml.sax.InputSource;

import ptolemy.actor.TypedIOPort;
import ptolemy.data.DoubleToken;
import ptolemy.data.IntToken;
import ptolemy.data.LongToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

/**
 * A ResultRecord that is in EML 2.0.0 format.
 */
public class Eml200DataSource extends ResultRecord
{

    //public TypedIOPort datasetPath;
    public FileParameter datasetPath;
    private org.ecoinformatics.util.DebugPrinter dbg;
    private String names[];
    private Type types[];
    private TableEntity tableentity;
    private Vector[] datavec;
    private String packagePath = "";
    private String datasetPathInitText = "Enter path to data package here.";     
    private int recordCount = 0;

    /**
     * Construct an actor with the given container and name.
     *
     * @param container The container.
     * @param name The name of this actor.
     * @exception IllegalActionException If the actor cannot be contained by the
     *                proposed container.
     * @exception NameDuplicationException If the container already has an actor
     *                with this name.
     * @since
     */
    public Eml200DataSource(CompositeEntity container, String name)
                    throws NameDuplicationException, IllegalActionException
    {
        super(container, name);
        dbg = org.ecoinformatics.util.DebugPrinter.getInstance();
        datasetPath = new FileParameter(this, "EML File");
        //datasetPath = new Parameter(this, "datasetPath",
        //    new StringToken(datasetPathInitText));
    }

    /**
     * Initialize the actor prior to running in the workflow. This reads the metadata
     * and configures the ports.
     *
     * @throws IllegalActionException
     */
    public void initialize() throws IllegalActionException
    {
        //String packagePath = datasetPath.asFile().getAbsolutePath();

        //parsePackage(packagePath);
        recordCount = 0;
    }

    /**
     * @return Description of the Returned Value
     * @exception IllegalActionException Description of Exception
     * @since
     */
    public boolean prefire() throws IllegalActionException
    {
        return super.prefire();
    }

    /**
     * Send a record's tokens over the ports on each fire event.
     *
     * @exception IllegalActionException If there is no director.
     */
    public void fire() throws IllegalActionException
    {
        super.fire();

        Attribute[] atts = tableentity.getAttributes();

            int i = recordCount;
            //dbg.print("Processing record: " + i, 2);
            Token[] vals = new Token[datavec[i].size()];
            for (int j = 0; j < datavec[i].size(); j++) {
                //dbg.print("        Processing attribute: " + j, 2);
                String name = atts[j].getName();
                String type = atts[j].getDataType();
                Object element = datavec[i].elementAt(j);
                TypedIOPort p = (TypedIOPort) this.getPort(name);

                //find the data type for each att
                if (type.equals(DataType.INT)) {
                    try {
                        vals[j] = new IntToken(new Integer((String) element)
                                        .intValue());
                    } catch (Exception e) {
                        vals[j] = new IntToken(0);
                    }
                } else if (type.equals(DataType.FLOAT)
                                || type.equals(DataType.DOUBLE)) {
                    vals[j] = new DoubleToken(new Double((String) element)
                                    .doubleValue());
                } else if (type.equals(DataType.LONG)) {
                    vals[j] = new LongToken(new Long((String) element)
                                    .longValue());
                } else if (type.equals(DataType.STR)) {
                    vals[j] = new StringToken((String) element);
                }
                //send the data
                p.send(0, vals[j]);
            }
          recordCount++;
    }

    /**
     * Callback for changes in attribute values.
     */
    public void attributeChanged(ptolemy.kernel.util.Attribute a)
                    throws ptolemy.kernel.util.IllegalActionException
    {
        if ((a.getName().equals("recordId") || a.getName().equals("endpoint")) &&
                        this.hasConnectionValues() &&
                        !(a.getContainer().getContainer() instanceof ResultTreeRoot)) {


        //if (a.getName().equals("EML File")) { //if the datasetPath token
                                              // changes, we need to change the
                                              // port
            //signature of the actor to match the new dataset
            //so we need to parse the attributes in the package and and a port
            //for each.
            //File file = ((FileParameter) a).asFile();
            //String packageFile = file.getAbsolutePath();
            //System.out.println("token is: " + packageFile);
            //if (packageFile != null) {

                //if (packageFile.equals(datasetPathInitText)) {
                    //don't parse. this is just the first time the actor
                    // has been drug to the canvas
                    //return;
                //}

                // Check to be sure that we only parse the file when a change is
                // made, and not when the values stay the same
                //if (!packagePath.equals(packageFile)) {
                    //parsePackage(packageFile);
                    //packagePath = packageFile;
                    Reader fullRecord = this.getFullRecord();
                    parsePackage(fullRecord);

                    Vector currentPortNames = getExistingPortNames();

                    for (int i = 0; i < names.length; i++) {
                        //create a new port for each attribute
                        try {
                            if (currentPortNames.contains(names[i].trim())) {
                                //set the type of the existing port
                                TypedIOPort currentPort = (TypedIOPort) this
                                                .getPort(names[i].trim());
                                currentPort.setTypeEquals(types[i]);
                            } else {
                                //create a new typed port and add it to this container
                                TypedIOPort p = new TypedIOPort(this, names[i],
                                                false, true);
                                p.setTypeEquals(types[i]);
                            }
                        } catch (ptolemy.kernel.util.NameDuplicationException nde) {
                            throw new IllegalActionException(
                                "One or more attributes has the " +
                                "same name.  Please correct this and try again.");
                        }
                    }
                //}
            //}
        }
    }

    /**
     * returns the names of the current ports
     */
    private Vector getExistingPortNames()
    {
        List l = this.portList();
        Vector v = new Vector();
        for (int i = 0; i < l.size(); i++) {
            TypedIOPort p = (TypedIOPort) l.get(i);
            v.add(p.getName().trim());
        }
        return v;
    }

    /**
     * parses the package and fills in the names and types arrays
     */
    private void parsePackage(String packageFile) throws IllegalActionException
    {
        File f = new File(packageFile);

        try { //parse the package for the names and types of the atts
            FileInputStream fis = new FileInputStream(f);
            parsePackage(fis);
        } catch (FileNotFoundException fnfe) {
            throw new IllegalActionException(
                            "Could not find EML file to parse. "
                                            + fnfe.getMessage());
        }
    }

    /**
     * parses the package and fills in the names and types arrays
     */
    private void parsePackage(InputStream eml) throws IllegalActionException
    {
        Eml200Parser eml2parser;

        try { //parse the package for the names and types of the atts
            dbg.print("creating parser", 2);
            eml2parser = new Eml200Parser();
            dbg.print("parsing...", 2);
            eml2parser.parse(eml);
            dbg.print("Done parsing", 2);
        } catch (Exception e) {
            throw new IllegalActionException("Error parsing the eml package: "
                            + e.getMessage());
        }

        tableentity = getData(eml2parser);
        processNamesAndTypes();
    }

    /**
     * parses the package and fills in the names and types arrays
     */
    private void parsePackage(Reader eml) throws IllegalActionException
    {
        Eml200Parser eml2parser;

        try { //parse the package for the names and types of the atts
            dbg.print("creating parser", 2);
            eml2parser = new Eml200Parser();
            dbg.print("parsing...", 2);
            eml2parser.parse(new InputSource(eml));
            dbg.print("Done parsing", 2);
        } catch (Exception e) {
            throw new IllegalActionException("Error parsing the eml package: "
                            + e.getMessage());
        }

        tableentity = getData(eml2parser);
        processNamesAndTypes();
    }

    /**
     * get the data based on the contents of the package
     *
     * @param parser the parser that has already parsed the package.
     */
    private TableEntity getData(Eml200Parser parser)
                    throws IllegalActionException
    {
        if (parser.getEntityCount() > 1) {
            throw new IllegalActionException(
                "Currently this parser only deals " +
                "with one entity.  Please use a package with only one entity.");
        } else if (parser.getEntityCount() == 0) { 
            throw new IllegalActionException(
                "There must be at least one entity in the EML package."); 
        }

        Hashtable entityhash = parser.getEntityHash();
        TableEntity entity = (TableEntity) entityhash.get(entityhash.keys()
                        .nextElement());
        dbg.print("entityhash: " + entityhash.toString(), 2);
        Hashtable filehash = parser.getDataFilesHash();
        dbg.print("filehash: " + filehash.toString(), 2);
        String fileurl = (String) filehash.get(filehash.keys().nextElement());
        dbg.print("Data URL is: " + fileurl, 2);
        
        InputStream filestream;
        StringBuffer dataSB = new StringBuffer();
        if (fileurl.startsWith("http://") || fileurl.startsWith("file://")) {
            // get the data from a URL
            try {
                URL url = new URL(fileurl);
                URLConnection conn = url.openConnection();
                filestream = url.openStream();
                InputStreamReader isr = new InputStreamReader(filestream);
                String type = conn.getContentType();
                char[] c = new char[1024];
                int bread = isr.read(c, 0, 1024);
                while (bread != -1) {
                    dataSB.append(c, 0, bread);
                    bread = isr.read(c, 0, 1024);
                }
                dbg.print("HTTP Data is: \n" + dataSB.toString(), 2);
            } catch (IOException ioe) {
                throw new IllegalActionException(
                                "There was an ioexception reading "
                                                + "the data file: "
                                                + ioe.getMessage());
            }
        } else if (fileurl.startsWith("ecogrid://")) {
            try {
                // create a ecogrid client object and get the full record from the
                // client
                EcogridQueryClient ecogridClient = new EcogridQueryClient(new URL(
                                this.getEndpoint()));
                ecogridClient.createEcoGridQueryLevelOnePortType();
                int start = fileurl.indexOf("/",11) + 1;
                dbg.print("start: " + start, 2);
                int end = fileurl.indexOf("/", start);
                if (end == -1) {
                    end = fileurl.length();
                }
                dbg.print("end: " + end, 2);
                String identifier = fileurl.substring(start, end);
                dbg.print("Get from EcoGrid: " + identifier, 2);
                byte[] record = ecogridClient.get(identifier);
                dbg.print("Bytes returned: " + record.length, 2);
                String recordString = new String(record);
                System.out.println(recordString);
                dataSB = new StringBuffer(recordString);
            } catch (Exception ee) {
                String error = ee.getMessage();
                dbg.print(error, 2);
            }
        } else {
            throw new IllegalActionException("Unsupported url protocol: "
                            + fileurl);
        }

        String data = dataSB.toString();

        DelimitedReader dr = new DelimitedReader(data,
                        entity.getAttributes().length, entity.getDelimiter(),
                        entity.getNumHeaderLines(), entity.getRecordDelimiter(), 
                        entity.getNumRecords());

        //datavec = new Vector[entity.getNumRecords()];
        datavec = dr.getTokenizedData(true);

        return entity;
    }

    /**
     * create the names array
     */
    private void processNamesAndTypes()
    {
        Attribute[] atts = tableentity.getAttributes();
        names = new String[atts.length];
        types = new Type[atts.length];
        for (int i = 0; i < atts.length; i++) {
            names[i] = atts[i].getName();
            String type = atts[i].getDataType();
            if (type.equals(DataType.INT)) {
                types[i] = BaseType.INT;
            } else if (type.equals(DataType.FLOAT)
                            || type.equals(DataType.DOUBLE)) {
                types[i] = BaseType.DOUBLE;
            } else if (type.equals(DataType.LONG)) {
                types[i] = BaseType.LONG;
            } else if (type.equals(DataType.STR)) {
                types[i] = BaseType.STRING;
            }
        }
    }
}

