/**

 *    '$RCSfile: EML2MetadataSpecification.java,v $'

 *

 *     '$Author: tao $'

 *       '$Date: 2004/07/28 21:54:35 $'

 *   '$Revision: 1.10 $'

 *

 *  For Details: http://kepler.ecoinformatics.org

 *

 * Copyright (c) 2004 The Regents of the University of California.

 * All rights reserved.

 *

 * Permission is hereby granted, without written agreement and without

 * license or royalty fees, to use, copy, modify, and distribute this

 * software and its documentation for any purpose, provided that the

 * above copyright notice and the following two paragraphs appear in

 * all copies of this software.

 *

 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN

 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY

 * OF SUCH DAMAGE.

 *

 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY

 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,

 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

 */



package org.ecoinformatics.seek.ecogrid;



import edu.ucsb.nceas.configxml.ConfigXML;

import edu.ucsb.nceas.configxml.exception.ElementNotFoundException;



import java.io.IOException;

import java.io.StringReader;

import java.util.Collections;

import java.util.Hashtable;

import java.util.Vector;



import org.apache.axis.types.URI;



import org.ecoinformatics.ecogrid.ANDType;

import org.ecoinformatics.ecogrid.ConditionType;

import org.ecoinformatics.ecogrid.LimitedXPathExpression;

import org.ecoinformatics.ecogrid.OperatorType;

import org.ecoinformatics.ecogrid.ORType;

import org.ecoinformatics.ecogrid.QueryType;

import org.ecoinformatics.ecogrid.QueryType_namespace;

import org.ecoinformatics.ecogrid.ResultsetType;

import org.ecoinformatics.ecogrid.ResultsetType_resultsetMetadata;

import org.ecoinformatics.ecogrid.ResultsetType_record;

import org.ecoinformatics.ecogrid.ResultsetType_record_returnField;

import org.ecoinformatics.ecogrid.ResultsetType_resultsetMetadata_recordStructure;

import org.ecoinformatics.ecogrid.ResultsetType_resultsetMetadata_recordStructure_returnField;



import org.ecoinformatics.seek.datasource.eml.eml2.Eml200DataSource;

import org.ecoinformatics.seek.ecogrid.exception.EcoGridException;

import org.ecoinformatics.util.DebugPrinter;

import org.ecoinformatics.util.Util;



import org.xml.sax.SAXException;

import org.xml.sax.ErrorHandler;

import org.xml.sax.XMLReader;

import org.xml.sax.InputSource;



import ptolemy.kernel.CompositeEntity;

import ptolemy.kernel.util.NameDuplicationException;

import ptolemy.kernel.util.IllegalActionException;

















/**

 *     This class is a plugin for eml200 namespace to transfer

 *     query group to ecogrid query type. It also provide a method to create

 *     quick search query

 *

 * @author Jing Tao

 */



public class EML2MetadataSpecification implements MetadataSpecificationInterface

{

  private DebugPrinter debugger = DebugPrinter.getInstance();

  private ConfigXML config;

  public static final String EML200NAMESPACE =

                                           "eml://ecoinformatics.org/eml-2.0.0";

  private static final int NAMESPACEARRAYLENGTH = 1;

  private static final String OPERATOR     = "LIKE";

  private static final String FILERFIELD   = "filerField";

  private static final String HTTP         = "http://%";

  private static final String FILE         = "file://%";

  private static final String ECOGRID      = "ecogrid://%";

  private static final String UNKNOWNTITLE = "unknownTitle";

  //private String recordTitle  = null;

  //private Vector entityName   = null;





  /**

   * Default constructor

   */

  public EML2MetadataSpecification()

  {



  }//Eml200EcoGridQueryTransfer



  /**

   * Set up the config

   * @param conf ConfigXML

   */

  public void setConfigure(ConfigXML conf)

  {

    this.config = conf;

  }//setConfigure



  /**

   * Method to create a quick query search

   * @param value String

   * @return QueryType

   */

  public QueryType getQuickSearchEcoGridQuery(String value)

                                         throws InvalidEcogridQueryException

  {

    //check the value

    if (value == null || value.trim().equals(""))

    {

      throw new InvalidEcogridQueryException("QuickSearch value should be set");

    }

    QueryType ecogridQuery = new QueryType();

    addNamespaceToQuery(ecogridQuery);

    //xpath for config file

    String searchPath  = "//" + QUICKPATH + "[@" + NAMESPACE + "='" +

                         EML200NAMESPACE + "']";

    String returnField = "//" + RETURNFIELD + "[@" + NAMESPACE + "='" +

                         EML200NAMESPACE + "']";

    String title       = "//" + QUERYTITLE + "[@" + NAMESPACE + "='" +

                         EML200NAMESPACE + "']";

    Vector searchPathVector  = null;

    Vector returnFieldVector = null;

    Vector titleVector       = null;

    try

    {

      searchPathVector  = config.getValuesForPath(searchPath);

      returnFieldVector = config.getValuesForPath(returnField);

      titleVector       = config.getValuesForPath(title);

    }

    catch(Exception e)

    {

      debugger.print("Couldn't get value from config file in " +

                    "Eml200EcoGridQueryTransfer.getQuickSearchEcoGridQuery", 2);

    }

    // search path is requried, so it need be checked

    if (searchPathVector == null || searchPathVector.isEmpty())

    {

      throw new InvalidEcogridQueryException("QuickSearch XPath should be " +

                                         "specified in ecogridConfigure.xml");

    }

    // setup search xpath

    ANDType topAND = new ANDType();



    // this will create a or type such as: title like 'a' or keyword like 'a'...

    ORType searchOR = new ORType();

    int searchPathNumber = searchPathVector.size();

    ConditionType[] searchValueCondtion = new ConditionType[searchPathNumber];

    for (int i=0; i<searchPathNumber; i++)

    {

      String searchPathStr = (String) searchPathVector.elementAt(i);

      ConditionType condition = generateConditionType(searchPathStr,

                                                     OPERATOR, value);

      searchValueCondtion[i]= condition;

    }//for

    searchOR.setCondition(searchValueCondtion);



    // this method will generated a filer, only show

    //dataset/dataTable/physical/distribution/online/url contians http:// or

    // file:// or ecogrid://

    ORType urlFiler = generateFiler();



    // create ORType array

    int size = 0;

    ORType[] orTypeArray = null;

    if (urlFiler != null)

    {

      size = 2;

      orTypeArray = new ORType[size];

      orTypeArray[0] = searchOR;

      orTypeArray[1] = urlFiler;



    }

    else

    {

      size = 1;

      orTypeArray = new ORType[size];

      orTypeArray[0] = searchOR;

    }



    topAND.setOR(orTypeArray);



    ecogridQuery.setAND(topAND);





    // set up returnfileds

    addReturnFieldToQuery(ecogridQuery, returnFieldVector);

    // set up query tile

    addTitleToQuery(ecogridQuery, titleVector);

    return ecogridQuery;

  }//getQuickSearchEcoGridQuery



  /**

   * Method to create a ecogrid query

   * @return QueryType

   */

  public QueryType getEcoGridQuery()

  {

    QueryType ecogridQuery = null;

    return ecogridQuery;

  }//getEcoGridQuery



  /**

   * This method will transfer ResultsetType java object to array of

   * ResultRecord java object. The ResultRecord object can be shown in kepler.

   * If the results is null or there is no record in the result, null will

   * be return

   * @param ResultsetType results   the result need to be transform

   * @param String endpoints  the search end point

   * @return ResultRecord[]   the resultrecord need be returned.

   */

  public ResultRecord[] transformResultset(ResultsetType results,

                                    String endpoint, CompositeEntity container)

                            throws SAXException, IOException, EcoGridException,

                               NameDuplicationException, IllegalActionException

  {

    Eml200DataSource[] resultRecordArray = null;

    if (results == null)

    {

      return resultRecordArray;

    }



    // get titlstring and entity name string from configure file

    String titleReturnFieldString  = null;

    String entityReturnFieldString = null;

    //xpath for config file

    String titlePath  = "//" + RETURNFIELD + "[@" + NAMESPACE + "='" +

                         EML200NAMESPACE + "' and @" + RETURNFIELDTYPE + "='" +

                         RETURNFIELDTITLE + "']";

    String entityPath = "//" + RETURNFIELD + "[@" + NAMESPACE + "='" +

                         EML200NAMESPACE + "' and @" + RETURNFIELDTYPE + "='" +

                         RETURNFIELDENTITY + "']";

    Vector titlePathVector  = null;

    Vector entityPathVector = null;

    try

    {

      titlePathVector  = config.getValuesForPath(titlePath);

      entityPathVector = config.getValuesForPath(entityPath);

    }

    catch(Exception e)

    {

      debugger.print("Couldn't get title and entity returnfield from config" +

                     "Eml200EcoGridQueryTransfer.transformResultset", 2);

      throw new EcoGridException(

      "Couldn't get title and entity returnfield from config" +e.getMessage());

    }

    // only choose the first one in vector as title returnfied or entityreturn

    // field

    titleReturnFieldString  = (String) titlePathVector.elementAt(0);

    entityReturnFieldString = (String) entityPathVector.elementAt(0);



    // transfer ResultType to a vector of eml2resultsetItem and

    // sorted the vector

    Vector resultsetItemList = transformResultsetType(results,

                             titleReturnFieldString, entityReturnFieldString);

    // transfer the sored vector (contains eml2resultsetitem object to an array

    // of ResultRecord

    int arraySize = resultsetItemList.size();

    resultRecordArray = new Eml200DataSource[arraySize];

    Hashtable titleList = new Hashtable();// This hashtable is for keeping track

                                          // if there is a duplicate title

    for (int i=0; i<arraySize; i++)

    {

      try

      {

        EML2ResultsetItem source = (EML2ResultsetItem)

                                    resultsetItemList.elementAt(i);

        String title      = source.getTitle();

        debugger.print("The title is " + title, 2);

        String id         = source.getId();

        debugger.print("The id is " + id, 2);

        Vector entityList = source.getEntityList();

        // if couldn't find id, skip this record

        if (id == null || id.trim().equals(""))

        {

          continue;

        }



        // if couldn't find title, assign a one to it -- <j>

        if (title == null || title.trim().equals(""))

        {

          title = "<" + i + ">";

        }

        if (titleList.containsKey(title))

        {

          title = title + " " + i;

        }

        titleList.put(title, title);

        Eml200DataSource newRecord = new Eml200DataSource(container, title);

        newRecord.setRecordId(id);

        newRecord.setEndpoint(endpoint);

        for (int j = 0; j < entityList.size(); j++)

        {

          String entityNameString = (String) entityList.elementAt(j);

          debugger.print("The entiy name will be " + entityNameString, 2);

          newRecord.addRecordDetail(entityNameString);



        }



        resultRecordArray[i] = newRecord;

      }

      catch(Exception e)

      {

        continue;

      }

    }//for

    return resultRecordArray;

  }





  /*

   * Method to add namespace into ecogrid query

   */

  private void addNamespaceToQuery(QueryType query)

                                      throws InvalidEcogridQueryException

  {

    if (query == null)

    {

      return;

    }

    // only search one name space, the array length is 1

    QueryType_namespace [] nameSpace =

                              new QueryType_namespace[NAMESPACEARRAYLENGTH];

    QueryType_namespace name = new QueryType_namespace();

    try

    {

      name.setValue(new URI(EML200NAMESPACE));

    }

    catch (Exception e)

    {

      throw new InvalidEcogridQueryException(e.getMessage());

    }

    //name.setValue(EML200NAMESPACE);

    nameSpace[0] = name;

    query.setNamespace(nameSpace);

  }//addNamespaceToQuery





  /*

   * Method to add return fields to ecogrid query

   */

  private void addReturnFieldToQuery(QueryType query, Vector returnField)

  {

    // check query and return field

    if ( query == null || returnField == null || returnField.isEmpty())

    {

      return;

    }

    // transfer the vector to a string array

    String[] fields = vectorToStringArray(returnField);

    // set the string array - return fields to ecogrid query

    query.setReturnfield(fields);

  }//addReturnFieldToQuery



  /*

   * Method to add query title into a query

   */

  private void addTitleToQuery(QueryType query, Vector titles)

  {

    if (query == null || titles == null || titles.isEmpty())

    {

      return;

    }

    //transfer the vector to a string array

    String[] titleArray = vectorToStringArray(titles);

    // set the sring array - titles to ecogrid query

    query.setTitle(titleArray);

  }//addTitleToQuery



  /*

   * Vector to string array.

   */

  private String[] vectorToStringArray(Vector vector)

  {

    String[] array = null;

    Vector vectorWithoutEmptyString = new Vector();

    if (vector == null|| vector.isEmpty())

    {

      return array;

    }

    // generate a new vector without empty string or null

    for (int i= 0; i< vector.size(); i++)

    {

      String obj = (String)vector.elementAt(i);

      if (obj !=null && !(obj.trim()).equals(""))

      {

        vectorWithoutEmptyString.add(obj);

      }

    }

    // copy the new vector without empty string to the array

    int length = vectorWithoutEmptyString.size();

    array = new String[length];

    //copy the vector to a string array

    for (int i =0; i<length; i++)

    {

      String str = (String) vectorWithoutEmptyString.elementAt(i);

      array[i]  = str;

    }

    return array;

  }//vectorToStringArray



  /*

   * Generate a condition type base on concept, operator, value

   */

  private ConditionType generateConditionType(String concept,

                                       String operator, String value)

                                       throws InvalidEcogridQueryException

  {

    // check the parameters



    if ( concept == null || concept.trim().equals(""))

   {

     throw new InvalidEcogridQueryException("Should set up concept value in " +

                                         "condition type in ecogrid query");

   }

   else if (operator == null || operator.trim().equals(""))

   {

      throw new InvalidEcogridQueryException("Should set up operator value in "+

                                         "condition type in ecogrid query");

   }

   else if (value == null || value.trim().equals(""))

   {

     throw new InvalidEcogridQueryException("Should set up search value in " +

                                        "condition type in ecogrid query");

    }



    // set up the limitedPathExpression (xpath)

    LimitedXPathExpression path = new LimitedXPathExpression(concept);



    // set up the operator

    OperatorType operatorType = null;

    try

    {

      operatorType = OperatorType.fromString(operator);

    }

    catch(IllegalStateException e)

    {

     throw new InvalidEcogridQueryException(e.getMessage());

    }



    //set up condition type

    ConditionType condition = new ConditionType();

    condition.setConcept(path);

    condition.setOperator(operatorType);

    condition.setValue(value);

    return condition;



  }



  /*

   * Method to generate a filer with or type only show

   *  dataset/dataTable/physical/distribution/online/url contians http:// or

   * file:// or ecogrid://

   */

   private ORType generateFiler()

   {

     ORType or = new ORType();

     String filerPath = "//" + FILERFIELD + "[@" + NAMESPACE + "='" +

         EML200NAMESPACE + "']";

     Vector filerVector = new Vector();

     String filerField = null;

     try {

       filerVector = config.getValuesForPath(filerPath);

       filerField = (String) filerVector.elementAt(0);

       debugger.print("The filer field in config is " + filerField, 2);

     }

     catch (Exception e) {

       debugger.print("The eror to get the filer field is " + e.getMessage(), 2);

       return null;

     }

     if (filerField == null || filerField.trim().equals("")) {

       return null;

     }

     ConditionType[] conditionArray = new ConditionType[3];

     try

     {

       ConditionType http = generateConditionType(filerField, OPERATOR, HTTP);

       ConditionType file = generateConditionType(filerField, OPERATOR, FILE);

       ConditionType eco = generateConditionType(filerField, OPERATOR, ECOGRID);

       conditionArray[0] = http;

       conditionArray[1] = file;

       conditionArray[2] = eco;

     }

     catch (Exception ee)

     {

       debugger.print("The eror to generate filer condition is " +

                      ee.getMessage(), 2);

       return null;

     }

     or.setCondition(conditionArray);

     return or;

   }









    /*

     * Method to transform array of AnyRecordType to array of EML2ResultsetItem

     */

     private Vector transformResultsetType(ResultsetType result,

                  String titleReturnFieldString, String entityReturnFieldString)

     {

       Vector itemList = new Vector();

       if (result == null)

       {

         return itemList;

       }

       ResultsetType_record[]records = result.getRecord();

       if (records == null)

       {

         return itemList;

       }



       ResultsetType_resultsetMetadata metadata = result.getResultsetMetadata();

       String titleId = getIdForGivenReturnField(titleReturnFieldString,

                                                 EML200NAMESPACE, metadata);

       debugger.print("The title id for eml2 is " + titleId, 2);

       String entityId = getIdForGivenReturnField(entityReturnFieldString,

                                                EML200NAMESPACE, metadata);

       debugger.print("The entity id for eml2 is " + entityId, 2);

       int arraySize = records.length;

       // transfer every records from source to a dest - EML2ResultsetItem obj

       for (int i=0; i<arraySize; i++)

       {

         ResultsetType_record currentRecord = records[i];

         if (currentRecord == null)

         {

           continue;

         }

         String docid = currentRecord.getIdentifier();

         debugger.print("The doc id after parsing resultset is " + docid, 2);

         // if couldn't find identifier, we don't need it

         if (docid == null || docid.trim().equals(""))

         {

           continue;

         }

         String recordTitle = null;

         Vector entityNameList = new Vector();

         ResultsetType_record_returnField [] recordReturnFieldList =

                            currentRecord.getReturnField();

         if (recordReturnFieldList == null )

         {

           continue;

         }

         int length = recordReturnFieldList.length;

         for (int j = 0; j < length; j++)

         {

           ResultsetType_record_returnField currentReturnField =

                                           recordReturnFieldList[j];

           if ( currentReturnField == null)

           {

             continue;

           }

           String returnFieldId    = currentReturnField.getId();

           String returnFieldValue = currentReturnField.getContent();

           if (returnFieldId != null && !returnFieldId.trim().equals(""))

           {

               if (titleId != null && returnFieldId.equals(titleId))

               {

                 debugger.print("The title after parsing is  " +

                                 returnFieldValue, 2);

                 recordTitle = returnFieldValue;

               }

               if (entityId != null && returnFieldId.equals(entityId))

               {

                 debugger.print("The original entity is " +returnFieldValue, 2);

                 returnFieldValue = replaceDotByDash(returnFieldValue);

                 entityNameList.add(returnFieldValue);

               }

           }



         }//for

         if (recordTitle == null )

         {

           recordTitle = UNKNOWNTITLE + i;

         }

         EML2ResultsetItem newItem = new EML2ResultsetItem(recordTitle, docid,

                                                             entityNameList);

         itemList.add(newItem);





      }//for

      Collections.sort(itemList, new EML2ResultsetItemComparator());

      return itemList;

    }







    /* A method get a id for given returnfiled and namespace in a metadata.

     * if more than one can be found, we only chose the first one

     */

    private String getIdForGivenReturnField(String givenReturnField,

                                            String givenNameSpace,

                                      ResultsetType_resultsetMetadata metadata)

    {

      String id = null;

      if ( givenReturnField == null || givenNameSpace == null || metadata == null)

      {

        return id;

      }



      ResultsetType_resultsetMetadata_recordStructure[] recordStructureList =

              metadata.getRecordStructure();

      if (recordStructureList == null)

      {

        return id;

      }

      int size = recordStructureList.length;

      for (int i= 0; i < size; i++)

      {

        ResultsetType_resultsetMetadata_recordStructure structure =

                                    recordStructureList[i];

        if (structure != null )

        {

          String nameSpace = structure.getNamespace();

          if (nameSpace != null && nameSpace.equals(givenNameSpace))

          {

            debugger.print("The target namespace " + nameSpace + " is found", 2);

            ResultsetType_resultsetMetadata_recordStructure_returnField[]

                            returnFieldList = structure.getReturnField();

            if (returnFieldList != null)

            {

              int length = returnFieldList.length;

              for (int j = 0; j < length; j++)

              {

                ResultsetType_resultsetMetadata_recordStructure_returnField

                    currentReturnField = returnFieldList[j];

                if (currentReturnField != null)

                {

                  String returnFieldName = currentReturnField.getName();

                  String returnFieldId   = currentReturnField.getId();

                  if (returnFieldName != null &&

                      returnFieldName.equals(givenReturnField))

                  {

                    id = returnFieldId;

                    if (id != null)

                    {

                      debugger.print("find the id " + id, 2);

                      break;

                    }

                  }

                }//if currentReturnField != null)

              }//for

            }//if returnFieldList !=null

          }//if namesapce !=null

        }//if structure != null

        // if already find the id, jump

        if (id != null)

        {

          break;

        }

      }//for

      debugger.print("the final id  for given namespace" + givenNameSpace +

                    " and given return field " + givenReturnField + " is " +

                     id, 2);

      return id;

    }



    /* A a method will replace a "." by "-" in a string */

    private String replaceDotByDash(String originalString)

    {

      String withoutDotString = null;

      if (originalString == null)

      {

        return withoutDotString;

      }



      if (originalString.indexOf(".")!= -1)

      {

        withoutDotString = originalString.replace('.', '-');



      }

      else

      {

        withoutDotString = originalString;

      }

      debugger.print("The string without dot is " + withoutDotString, 2);

      return withoutDotString;

    }





  }//Eml200EcoGridQeryTransfer

