/**
 *    '$RCSfile: MetadataSpecificationInterface.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/05/06 20:20:20 $'
 *   '$Revision: 1.5 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.ecogrid;

import edu.ucsb.nceas.configxml.ConfigXML;

import java.io.IOException;

import org.ecoinformatics.ecogrid.QueryType;
import org.ecoinformatics.ecogrid.ResultsetType;
import org.ecoinformatics.seek.ecogrid.exception.EcoGridException;
import org.xml.sax.SAXException;

import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.IllegalActionException;



/**
 * <p>Title:PlugInQueryTransferInterface </p>
 * <p>Description: This is a plugin interface and it defines a all methods any
 *   metadata type should implement into order to search ecogrid service
*    and parse the results</p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author not attributable
 * @version 1.0
 */

public interface MetadataSpecificationInterface
{
  //Constant
  public static final String RETURNFIELD        = "returnField";
  public static final String RETURNFIELDTYPE    = "type";
  public static final String RETURNFIELDTITLE   = "title";
  public static final String RETURNFIELDENTITY  = "entityName";
  public static final String NAMESPACE          = "namespace";
  public static final String QUICKPATH          = "quickSearchPath";
  public static final String QUERYTITLE         = "queryTitle";
  public static final String ENDPOINT           = "endPoint";
  /**
   *  Method to set a config object
   * @param config ConfigXML  the config will be set
   */
  public void setConfigure(ConfigXML config);

  /**
   * This method will return a quick search query base on the given value
   * @param value String
   * @return QueryType
   */
  public QueryType getQuickSearchEcoGridQuery(String value)
                                          throws InvalidEcogridQueryException;

  /**
   * This method will transfer a query group into ecogrid query base on
   * different namespace
   * @return QueryType
   */
  public QueryType getEcoGridQuery();

  /**
   * This method will transfer ResultsetType java object to array of
   * ResultRecord java object. The ResultRecord object can be shown in kepler
   * @param ResultsetType results   the result need to be transform
   * @param String endpoints  the search ends points
   * @return ResultRecord[]   the resultrecord need be returned.
   */
  public ResultRecord[] transformResultset(ResultsetType results,
                                 String endpoint, CompositeEntity container)
                         throws SAXException, IOException, EcoGridException,
                          NameDuplicationException, IllegalActionException;


}
