/**
 *    '$RCSfile: TabbedLibraryPane.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/08/12 18:40:27 $'
 *   '$Revision: 1.18 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.ecogrid;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTree;

import javax.swing.ActionMap;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.tree.TreeModel;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import ptolemy.gui.CancelException;
import ptolemy.gui.MessageHandler;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.NamedObj;
import ptolemy.kernel.Entity;
import ptolemy.vergil.tree.EntityTreeModel;
import ptolemy.vergil.tree.PTree;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.ComponentEntity;
import ptolemy.moml.MoMLParser;
import ptolemy.moml.ErrorHandler;

import java.util.Vector;
import java.util.List;
import java.util.EmptyStackException;
import java.util.Stack;

import edu.ucsb.nceas.configxml.ConfigXML;

/**
 * A simple pane used to display the contents of the library tree using a tabbed pane
 * to provide additional resources in the library.
 *
 * @author  Matt Jones
 * @version $Id: TabbedLibraryPane.java,v 1.18 2004/08/12 18:40:27 berkley Exp $
 * @since Kepler 1.0
 */
public class TabbedLibraryPane extends LibraryPane
{
    /**
     * Construct a new library pane for displaying the tree of actors and other
     * resourses that can be dragged to the graph editor.
     *
     * @param _libraryModel the model containing the library to be displayed
     */
    public TabbedLibraryPane(EntityTreeModel _libraryModel)
    {
        super();

        this.setLayout(new BorderLayout());
        _library = new PTree(_libraryModel);
        _library.setRootVisible(false);
        _library.setBackground(BACKGROUND_COLOR);
        _libraryScrollPane = new JScrollPane(_library);
        JPanel librarySearchPanel = new JPanel();
        librarySearchPanel.setLayout(topLayout);
        librarySearchPanel.setMinimumSize(new Dimension(200, 500));
        librarySearchPanel.setPreferredSize(new Dimension(200, 500));
        //create and add the search panel to the library
        initSearchPartPanel();
        _libraryScrollPane.setPreferredSize(new Dimension(200, 400));
        librarySearchPanel.add(searchPartPanel, BorderLayout.NORTH);
        librarySearchPanel.add(_libraryScrollPane, BorderLayout.CENTER);

        searchPartPanel.setPreferredSize(new Dimension(200, 100));
        searchPartPanel.setMinimumSize(new Dimension(100, 75));
        

        _libraryTabPane = new JTabbedPane(JTabbedPane.TOP);
        _libraryTabPane.add("Actors", librarySearchPanel);

        try {
            DatasetPanel _datasetPanel = new DatasetPanel();

            SearchScopeController controller = new SearchScopeController();
            Vector searchScopeVector = controller.getSearchScopeVector();
            _datasetPanel.setSearchScope(searchScopeVector);

            _libraryTabPane.add("Data", _datasetPanel);
        } catch (Exception e) {
            try {
                MessageHandler.warning("Could not create data pane.", e);
            } catch (Exception ce) {
                // Do nothing
            }
        }

        //THE FOLLOWING 2 LINES ARE A HACK
        //When the momlparser hits an error in the moml, it rebuilds the model
        //which in turn collapses the entire library JTree.  These two lines
        //presearch the tree which causes the parse errors to get thrown before
        //the first search so the tree won't be collapsed the first time you
        //search.
        LibrarySearch ls = new LibrarySearch(_library);
        ls.search("this is a search that you will not find");

        this.add(_libraryTabPane, BorderLayout.CENTER);
    }

    /*
     * Initialize the search panel
     */
    private void initSearchPartPanel() 
    {
        BoxLayout searchPartLayout = new BoxLayout(searchPartPanel,
                        BoxLayout.Y_AXIS);
        searchPartPanel.setLayout(searchPartLayout);
        //add a blank panel as border
        JPanel topBlankPanel = new JPanel();
        topBlankPanel.add(Box.createVerticalStrut(VERTICALEDGEGAP));
        searchPartPanel.add(topBlankPanel);

        // the panel contains label - Quick Search
        JPanel quickSearchLabelPanel = new JPanel();
        quickSearchLabelPanel.setLayout(new BoxLayout(quickSearchLabelPanel,
                        BoxLayout.X_AXIS));
        quickSearchLabelPanel.add(Box.createHorizontalStrut(EDGEGAP));
        quickSearchLabel = new JLabel(QUICKSEARCHLABEL);
        quickSearchLabelPanel.add(quickSearchLabel);
        quickSearchLabelPanel.add(Box.createHorizontalGlue());
        // add the quickSearchLabelPanel to searchPartPanel
        searchPartPanel.add(quickSearchLabelPanel);

        //add a bank panel between quicksearch panel and textFieldAndButton
        // panel
        JPanel blankPanelUnderQuickSearch = new JPanel();
        blankPanelUnderQuickSearch.add(Box
                        .createVerticalStrut(VERTICALCOMPONENTGAP));
        searchPartPanel.add(blankPanelUnderQuickSearch);

        // the panel contains the textfield and go button, it is has a fixed
        // height
        JPanel textFieldAndButtonPanel = new JPanel();
        textFieldAndButtonPanel.setLayout(new BoxLayout(
                        textFieldAndButtonPanel, BoxLayout.X_AXIS));
        textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
        searchValueField = new JTextField();
        textFieldAndButtonPanel.add(searchValueField);
        textFieldAndButtonPanel.add(Box.createHorizontalStrut(COMPONENTGAP));
        // quickSearchAction

        quickSearchButton = new JButton(GO);
        searchButtonHandler = new SearchButtonActionHandler();
        quickSearchButton.addActionListener(searchButtonHandler);
        textFieldAndButtonPanel.add(quickSearchButton);
        textFieldAndButtonPanel.add(Box.createHorizontalStrut(EDGEGAP));
        textFieldAndButtonPanel.add(Box.createHorizontalGlue());

        //add the textFieldAndButtonPanel into searchPartPanel
        searchPartPanel.add(textFieldAndButtonPanel);

        //add a bank panel between quicksearch panel and textFieldAndButton
        // panel
        JPanel blankPanelUnderTextField = new JPanel();
        blankPanelUnderTextField.add(Box
                        .createVerticalStrut(VERTICALCOMPONENTGAP));
        searchPartPanel.add(blankPanelUnderTextField);

        //add a bank panel between quicksearch panel and textFieldAndButton
        // panel
        JPanel blankPanelUnderAdvanceSearch = new JPanel();
        blankPanelUnderAdvanceSearch.add(Box
                        .createVerticalStrut(VERTICALEDGEGAP));
        searchPartPanel.add(blankPanelUnderAdvanceSearch);
    }

    /**
     * A factory that creates the library panel for the editors.
     */
    public static class Factory extends LibraryPaneFactory
    {
        /** Create an factory with the given name and container.
         *  @param container The container.
         *  @param name The name of the entity.
         *  @exception IllegalActionException If the container is incompatible
         *   with this attribute.
         *  @exception NameDuplicationException If the name coincides with
         *   an attribute already in the container.
         */
        public Factory(NamedObj container, String name)
                throws IllegalActionException, NameDuplicationException
        {
            super(container, name);
        }

        /** Create a library pane that displays the given library of actors.
         *
         *  @param _libraryModel the model containing the actor library
         *  @return A new LibraryPane that displays the library
         */
        public LibraryPane createLibraryPane(EntityTreeModel _libraryModel)
        {
                TabbedLibraryPane pane = new TabbedLibraryPane(_libraryModel);
                return pane;
        }
    }

    /**
     * class to handle the search button
     */
    public class SearchButtonActionHandler implements ActionListener
    {
      public void actionPerformed(ActionEvent e)
      {
        LibrarySearch ls = new LibrarySearch(_library);
        ls.search(searchValueField.getText());
      }
    }

    /**
     * class to search the library
     */
    public class LibrarySearch
    {
      private JTree lib; //the library to search
      private Stack path = new Stack(); //the stack to create the return paths with
      private Vector results = new Vector();

      /**
       * constructor
       */
      public LibrarySearch(JTree library)
      {
        lib = library;
      }

      /**
       * search for val in the library
       */
      public void search(String val)
      {
        ErrorHandler handler = ptolemy.moml.MoMLParser.getErrorHandler();
        //temporarily disable error handling
        MoMLParser.setErrorHandler(new ErrorSkipper());
        TreeModel model = lib.getModel();
        findval(val, model); //create the results vector
        TreePath[] tp = createTreePathArray();
        lib.clearSelection(); //clear the current selections
        collapseAll();
        for(int i=0; i<tp.length; i++)
        {
          lib.addSelectionPath(tp[i]);
        }
        //reinstate the rightful errorhandler
        MoMLParser.setErrorHandler(handler);
      }

      /**
       * collapses the tree
       */
      private void collapseAll()
      {
        int count = lib.getRowCount();
        for(int i=count - 1; i>=0; i--)
        { //collapse in reverse order or else the outer nodes get collapsed
          //before the inner ones.
          if(!lib.isCollapsed(i))
          {
            lib.collapseRow(i);
          }
        }
      }

      /**
       * look for the val in the TreeModel
       */
      private void findval(String val, TreeModel model)
      {
        Object o = model.getRoot(); //start from the root
        path.push(o);
        for(int i=0; i<model.getChildCount(o); i++)
        {
          Object child = model.getChild(o, i);
          path.push(child);
          if(child instanceof CompositeEntity)
          {
            String name = ((CompositeEntity)child).getName();
            if(compareVals(name, val))
            { //this is what we are looking for...save the result
              results.add(createPathFromStack());
            }
            //go to the other findval method if we are looking in a CompositeEntity
            findval(val, (CompositeEntity)child);
          }
          else
          {
            System.out.println("this is not a compentity: " + child.toString());
          }
        }
      }

      /**
       * look for val recursively inside entity
       */
      private void findval(String val, CompositeEntity entity)
      {
        List entityList = entity.entityList();

        for(int i=0; i<entityList.size(); i++)
        {
          Entity e = (Entity)entityList.get(i);
          String name = e.getName();
          path.push(e);

          if(e instanceof CompositeEntity)
          { //recurse into the tree
            if(compareVals(name, val))
            { //this is what we are looking for...save the result
              results.add(createPathFromStack());
            }

            findval(val, (CompositeEntity)e); //keep searching
          }
          else
          { //leaf node
            if(compareVals(name, val))
            { //this is what we are looking for...return the result
              results.add(createPathFromStack());
            }

            if(!path.empty())
            { //remove the leaf node we just looked at
              path.pop();
            }
          }
        }

        if(!path.empty())
        { //remove the composite node we are currently searching
          path.pop();
        }
      }

      /**
       * compares two strings and returns true if they match
       */
      private boolean compareVals(String val1, String val2)
      {
        val1 = val1.toLowerCase(); //make the comparison case insensitive
        val2 = val2.toLowerCase();
        if(val1.indexOf(val2) != -1)
        {
          return true;
        }
        return false;
      }

      /**
       * return the path as a TreePath
       */
      private TreePath createPathFromStack()
      {
        return new TreePath(path.toArray());
      }

      /**
       * create an array of treepaths
       */
      private TreePath[] createTreePathArray()
      {
        TreePath[] tm = new TreePath[results.size()];
        for(int i=0; i<results.size(); i++)
        {
          tm[i] = (TreePath)results.elementAt(i);
        }
        return tm;
      }
    }

    /**
     * errorhandler that ignore all errors and continues parsing.
     */
    private class ErrorSkipper implements ErrorHandler
    {
      public void enableErrorSkipping(boolean b)
      {//yeah, we're skipping everything whether you tell us to or not
      }

      public int handleError(java.lang.String element, NamedObj context,
        java.lang.Throwable exception)
      {
        return ErrorHandler.CONTINUE; //just keep it moving
      }
    }



    ///////////////////////////////////////////////////////////////////
    ////                         private variables                 ////

    // FIXME: Comments are needed on all these.
    // FIXME: Need to be in alphabetical order.

    // NOTE: should be somewhere else?
    // Default background color is a light grey.
    private static Color BACKGROUND_COLOR = new Color(0xe5e5e5);

    private JTree _library;
    private JScrollPane _libraryScrollPane;
    private JScrollPane _dataScrollPane;
    private JTabbedPane _libraryTabPane;
    private JPanel searchPartPanel = new JPanel();
    private JLabel quickSearchLabel = null;
    private JTextField searchValueField = null;
    private Vector searchScopeVector;
    private JButton quickSearchButton = null;
    private SearchButtonActionHandler searchButtonHandler = null;
    private InputMap imap;
    private ActionMap amap;

    //constants
    private static final int EDGEGAP = 10;//GAP from container to component
    private static final int COMPONENTGAP = 6;//GAP from compent to component
    private static final int VERTICALEDGEGAP = 6;//GAP from container to
                                                 // component
    private static final int VERTICALCOMPONENTGAP = 3;//GAP from component
                                                      // tocompent
    private static final String QUICKSEARCHLABEL = "Quick Search";
    private static final String GO = "Go";
    private static final String QUICKSEARCHACTION = "quickSearchAction";

    private BorderLayout topLayout = new BorderLayout();
}
