/**
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.sdm.spa;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.IntToken;
import ptolemy.data.Token;
import ptolemy.data.type.BaseType;
import ptolemy.data.expr.Parameter;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

public class ExtractSequence
    extends TypedAtomicActor {

  public ExtractSequence(CompositeEntity aContainer, String aName) throws
      IllegalActionException, NameDuplicationException {

    super(aContainer, aName);

    portOutput = new TypedIOPort(this, "Output", false, true);
    portOutput.setTypeEquals(BaseType.STRING);
    portInput = new TypedIOPort(this, "Input", true, false);
    portInput.setTypeEquals(BaseType.STRING);
    portStart = new TypedIOPort(this, "Start", true, false);
    portInput.setTypeEquals(BaseType.INT);
    portEnd = new TypedIOPort(this, "End", true, false);
    portInput.setTypeEquals(BaseType.INT);
  }

  ///////////////////////////////////////////////////////////////////
  ////		 ports and parameters		                 ////

  public TypedIOPort portOutput;
  public TypedIOPort portInput;
  public TypedIOPort portStart;
  public TypedIOPort portEnd;

  ///////////////////////////////////////////////////////////////////
  ////			public methods				 ////

  /** Fire the actor.
   *  @exception IllegalActionException If there is no director.
   */
  public void fire() throws IllegalActionException {
    String strIn = ( (StringToken) portInput.get(0)).stringValue();
    _debug("Process:" + strIn);

    int iStart = ( (IntToken) portStart.get(0)).intValue();
    int iEnd = ( (IntToken) portEnd.get(0)).intValue();

    if (iStart < 1) {
      iStart = 1;
    }
    else if (iStart > strIn.length()) {
      iStart = strIn.length();
    }

    if (iEnd < iStart) {
      iEnd = iStart;
    }
    else if (iEnd > strIn.length()) {
      iEnd = strIn.length();
    }

    _debug("Output from " + iStart + " to " + iEnd + ": " +
           strIn.substring(iStart - 1, iEnd));
    portOutput.broadcast(new StringToken(strIn.substring(iStart - 1, iEnd)));
  }
}