/**
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.sdm.spa;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.StringToken;
import ptolemy.data.IntToken;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

public class StringAggregate
    extends TypedAtomicActor {

  public StringAggregate(CompositeEntity aContainer, String aName) throws
      IllegalActionException, NameDuplicationException {

    super(aContainer, aName);

    portOutput = new TypedIOPort(this, "portOutput", false, true);
    portOutput.setTypeEquals(BaseType.STRING);
    portInput = new TypedIOPort(this, "portInput", true, false);
    portInput.setTypeEquals(BaseType.STRING);
    portNum = new TypedIOPort(this, "portNum", true, false);
    portNum.setTypeEquals(BaseType.INT);
  }

  ///////////////////////////////////////////////////////////////////
  ////		 ports and parameters				  ////

  public TypedIOPort portOutput;
  public TypedIOPort portInput;
  public TypedIOPort portNum;

  ///////////////////////////////////////////////////////////////////
  ////			public methods			         ////

  /** Fire the actor.
   *  @exception IllegalActionException If there is no director.
   */
  public void fire() throws IllegalActionException {
    int num = ( (IntToken) portNum.get(0)).intValue();
    _debug("Process:" + num);

    StringBuffer buf = new StringBuffer();

    for (int i = 0; i < num; i++) {
      StringToken tokIn = (StringToken) portInput.get(0);
      buf = buf.append(tokIn.stringValue());
    }
    portOutput.broadcast(new StringToken(buf.toString()));
  }
}