/**
 *  '$RCSfile: DataQueryActionTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: Jing Tao
 *    Release: @release@
 *
 *   '$Author: tao $'
 *     '$Date: 2004/08/13 01:01:38 $'
 * '$Revision: 1.2 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.ecoinformatics.seek.dataquery;


import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import java.io.FileReader;
import java.io.FileNotFoundException;
import java.util.Hashtable;
import java.util.Vector;

import org.ecoinformatics.seek.datasource.eml.eml2.Eml200Parser;
import org.ecoinformatics.seek.datasource.TableEntity;
import org.ecoinformatics.seek.dataquery.DataQueryAction;
import org.ecoinformatics.util.DebugPrinter;

import org.xml.sax.InputSource;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.net.URLConnection;
import java.net.URL;
import org.ecoinformatics.util.DelimitedReader;
import org.ecoinformatics.seek.dataquery.DBTablesGenerator;



/**
 *
 */

public class DataQueryActionTest extends TestCase
{
  //constant
  private final static String EMLFILEPATH  = "./lib/testdata/eml/eml-sample.xml";
  private final static String DATAFILEPATH = "./lib/testdata/eml/sample.dat";
  private DebugPrinter dbg = null;
  private Eml200Parser parser = new Eml200Parser();
  private TableEntity entity  = null;
  public DataQueryActionTest(String name)
  {
    super(name);
    dbg = DebugPrinter.getInstance();
  }


  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
    //parse the eml document and get entity info
    FileReader reader = null;

    try
    {
      reader = new FileReader(EMLFILEPATH);
      InputSource source = new InputSource(reader);
      parser.parse(source);
    }
    catch(FileNotFoundException e)
    {
      dbg.print("couldn't find eml doc " + EMLFILEPATH, 2);
    }
    catch(Exception ee)
    {
      System.out.println("The error in parsing eml document");
      dbg.print("eror in parsing eml document",2);
    }
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    //suite.addTest(new DataQueryActionTest("initialize"));
    suite.addTest(new DataQueryActionTest("actionPerformedNontextTableTest"));
    //suite.addTest(new DataQueryActionTest("actionPerformedTextTableTest"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }

  /**
   * A test for main method actionPerformed base on non-textTablein DataQueryAction
   */
  public void actionPerformedNontextTableTest()
  {
    // create a hsql table and insert data
    try
    {
      Vector [] dataVector = getData();
      DBTablesGenerator tableGenerator = new DBTablesGenerator(entity,
                                                         dataVector);
      String tableName = entity.getName();
      String sql = "SELECT * FROM " + tableName;
      DataQueryAction query = new DataQueryAction();
      query.setSQL(sql);
      query.actionPerformed(null);
    }
    catch (Exception e)
    {
      dbg.print("Eror to create tables in db " + e.getMessage(), 2);
      assertTrue(1 == 2);
    }


  }//actionPerformedNontextTableTest

  /**
  * A test for main method actionPerformed base on non-textTablein DataQueryAction
  */
 public void actionPerformedTextTableTest()
 {
   // create a hsql table and insert data
   try
   {
     Hashtable entityhash = parser.getEntityHash();
     entity = (TableEntity) entityhash.get(entityhash.keys()
        .nextElement());
     DBTablesGenerator tableGenerator = new DBTablesGenerator(entity,
                                                        DATAFILEPATH);
     String tableName = entity.getName();
     String sql = "SELECT * FROM " + tableName;
     DataQueryAction query = new DataQueryAction();
     query.setSQL(sql);
     query.actionPerformed(null);
   }
   catch (Exception e)
   {
     dbg.print("Eror to create tables in db " + e.getMessage(), 2);
     assertTrue(1 == 2);
   }


 }//actionPerformedNontextTableTest


  private Vector[] getData() throws Exception
  {
    if (parser.getEntityCount() > 1)
    {
      throw new Exception("Currently this parser only deals " +
                "with one entity.  Please use a package with only one entity.");
    }
    else if (parser.getEntityCount() == 0)
    {
      throw new Exception(
                     "There must be at least one entity in the EML package.");
    }

    Hashtable entityhash = parser.getEntityHash();
    entity = (TableEntity) entityhash.get(entityhash.keys()
        .nextElement());
    dbg.print("entityhash: " + entityhash.toString(), 2);
    Hashtable filehash = parser.getDataFilesHash();
    dbg.print("filehash: " + filehash.toString(), 2);
    String fileurl = (String) filehash.get(filehash.keys().nextElement());
    dbg.print("Data URL is: " + fileurl, 2);

    InputStream filestream;
    StringBuffer dataSB = new StringBuffer();
    if (fileurl.startsWith("http://") || fileurl.startsWith("file://"))
    {
      // get the data from a URL
      try
      {
        URL url = new URL(fileurl);
        URLConnection conn = url.openConnection();
        filestream = url.openStream();
        InputStreamReader isr = new InputStreamReader(filestream);
        String type = conn.getContentType();
        char[] c = new char[1024];
        int bread = isr.read(c, 0, 1024);
        while (bread != -1)
        {
          dataSB.append(c, 0, bread);
          bread = isr.read(c, 0, 1024);
        }
        dbg.print("HTTP Data is: \n" + dataSB.toString(), 2);
      }
      catch (IOException ioe) {
        throw new Exception(
            "There was an ioexception reading "
            + "the data file: "
            + ioe.getMessage());
      }

    }
    else
    {
      throw new Exception("Unsupported url protocol: "
                          + fileurl);
    }

    String data = dataSB.toString();

    DelimitedReader dr = new DelimitedReader(data,
                                             entity.getAttributes().length,
                                             entity.getDelimiter(),
                                             entity.getNumHeaderLines(),
                                             entity.getRecordDelimiter(),
                                             entity.getNumRecords());
    Vector[] datavec = dr.getTokenizedData(true);
    return datavec;
  }

}//DataQueryActionTest


