/**

 *  '$RCSfile: SearchScopeTest.java,v $'

 *  Copyright: 2000 Regents of the University of California and the

 *              National Center for Ecological Analysis and Synthesis

 *    Authors: @authors@

 *    Release: @release@

 *

 *   '$Author: berkley $'

 *     '$Date: 2004/04/28 18:44:16 $'

 * '$Revision: 1.2 $'

 *

 * This program is free software; you can redistribute it and/or modify

 * it under the terms of the GNU General Public License as published by

 * the Free Software Foundation; either version 2 of the License, or

 * (at your option) any later version.

 *

 * This program is distributed in the hope that it will be useful,

 * but WITHOUT ANY WARRANTY; without even the implied warranty of

 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the

 * GNU General Public License for more details.

 *

 * You should have received a copy of the GNU General Public License

 * along with this program; if not, write to the Free Software

 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

 */



package test.org.ecoinformatics.seek.ecogrid;



import edu.ucsb.nceas.configxml.ConfigXML;

import junit.framework.Test;

import junit.framework.TestCase;

import junit.framework.TestResult;

import junit.framework.TestSuite;



import java.util.Vector;



import org.ecoinformatics.seek.ecogrid.SearchScope;

import org.ecoinformatics.seek.ecogrid.exception.

                                          NoMetadataSpecificationClassException;

import org.ecoinformatics.seek.ecogrid.exception.NoSearchEndPointException;

import org.ecoinformatics.seek.ecogrid.exception.

                                              NULLEcogridConfigurationException;

import org.ecoinformatics.seek.ecogrid.exception.NULLSearchNamespaceException;

import org.ecoinformatics.util.DebugPrinter;

import org.ecoinformatics.util.Util;





/**

 * <p>Title: SearchScopeTest</p>

 * <p>Description: </p>

 * <p>Copyright: Copyright (c) 2004</p>

 * <p>Company: </p>

 * @author not attributable

 * @version 1.0

 */



public class SearchScopeTest  extends TestCase

{

  private static final String ENDPOINT  = "http://dev.nceas.ucsb.edu";

  private static final String NAMESPACE = "eml://ecoinformatics.org/eml-2.0.0";

  private static final String CLASSNAME =

                   "org.ecoinformatics.seek.ecogrid.EML2MetadataSpecification";

  ConfigXML config = null;

  String namespace = null;

  Vector endpoints = null;

  SearchScope searchScope = null;



  public SearchScopeTest(String name)

  {

    super(name);

    try

    {

      config = new ConfigXML("lib/ecogridConfigure.xml");

    }

    catch (Exception e)

    {

       fail("Error to initial config file: " + e.getMessage());

    }

    namespace = NAMESPACE;

    endpoints = new Vector();

    endpoints.addElement(ENDPOINT);



  }





  /**

   * Establish a testing framework by initializing appropriate objects

   */

  public void setUp()

  {

  }



  /**

   * Release any objects after tests are complete

   */

  public void tearDown()

  {

  }



  /**

   * Create a suite of tests to be run together

   */

  public static Test suite()

  {

    TestSuite suite = new TestSuite();

    suite.addTest(new SearchScopeTest("initialize"));

    suite.addTest(new SearchScopeTest("constructorThrowNULLSearchNamespace"));

    suite.addTest(new SearchScopeTest("constructorThrowNULLConfiguration"));

    suite.addTest(new SearchScopeTest("constructorThrowNoMetadataClass"));

    suite.addTest(new SearchScopeTest("constructorThrowNoEndPoint"));

    suite.addTest(new SearchScopeTest("getNamespaceTest"));

    suite.addTest(new SearchScopeTest("getMetadataSpecificationTest"));

    suite.addTest(new SearchScopeTest("getEndPointsTest"));

    return suite;

  }



  /**

   * Run an initial test that always passes to check that the test

   * harness is working.

   */

  public void initialize()

  {

    assertTrue(1 == 1);

  }



  /**

   * Method to testing construct to throw a null search name space exception

   */

  public void constructorThrowNULLSearchNamespace()

  {

    boolean flag = false;

    namespace = null;

    try

    {

      searchScope = new SearchScope(namespace, config, endpoints);

    }

    catch (Exception e)

    {

      if (e instanceof NULLSearchNamespaceException)

      {

        flag = true;

      }

    }

    assertTrue(flag);

  }



  /**

   * Method to testing construct to throw a null configureation exception

   */

  public void constructorThrowNULLConfiguration()

  {

    boolean flag = false;

    config = null;

    try

    {

      searchScope = new SearchScope(namespace, config, endpoints);

    }

    catch (Exception e)

    {

      if (e instanceof NULLEcogridConfigurationException)

      {

        flag = true;

      }

    }

    assertTrue(flag);

  }



  /**

  * Method to testing construct to throw a null metadata specification

  */

 public void constructorThrowNoMetadataClass()

 {

   boolean flag = false;

   namespace = "hello";

   try

   {

     searchScope = new SearchScope(namespace, config, endpoints);

   }

   catch (Exception e)

   {



     if (e instanceof NoMetadataSpecificationClassException)

     {

       flag = true;

     }

   }

   assertTrue(flag);

 }



 /**

 * Method to testing construct to throw a no end points exception

 */

public void constructorThrowNoEndPoint()

{

  boolean flag = false;

  endpoints = null;

  try

  {

    searchScope = new SearchScope(namespace, config, endpoints);

  }

  catch (Exception e)

  {

    if (e instanceof NoSearchEndPointException)

    {

      flag = true;

    }

  }

  assertTrue(flag);

}





 /**

  * Method to testing getNamespace method

  */

 public void getNamespaceTest()

 {

   boolean flag = false;

   try

   {

     initInstance();

   }

   catch (Exception e)

   {

     flag = false;

   }

   if ( searchScope.getNamespace().equals(NAMESPACE))

   {

     flag = true;

   }

   assertTrue(flag);

 }



 /**

  * Test the getMetadataSpecification method

  */

 public void getMetadataSpecificationTest()

 {

   boolean flag = false;

   try

  {

    initInstance();

  }

  catch (Exception e)

  {

    flag = false;



  }

  if ( searchScope.getMetadataSpecification().

       getClass().getName().equals(CLASSNAME))

  {

    flag = true;

  }

  assertTrue(flag);

 }



 /**

 * Test the getEndPonts method

 */

public void getEndPointsTest()

{

  boolean flag = false;

  try

 {

   initInstance();

 }

 catch (Exception e)

 {

   flag = false;

 }

 Vector url = searchScope.getEndPoints();

 String urlString = (String)url.elementAt(0);

 if (urlString.equals(ENDPOINT))

 {

   flag = true;

 }

 assertTrue(flag);

}





 private void initInstance()throws Exception

 {

   try

  {

   searchScope = new SearchScope(namespace, config, endpoints);

  }

  catch (Exception e)

  {

    throw e;

  }

 }





}

