/**
 *    '$RCSfile: QuickSearchAction.java,v $'
 *
 *     '$Author: berkley $'
 *       '$Date: 2004/08/03 16:43:25 $'
 *   '$Revision: 1.11 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.ecogrid;

import edu.ucsb.nceas.configxml.ConfigXML;

import java.awt.event.ActionEvent;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.JTextField;

import ptolemy.kernel.CompositeEntity;
import org.ecoinformatics.ecogrid.QueryType;
import org.ecoinformatics.util.DebugPrinter;
import org.ecoinformatics.util.Util;
import org.ecoinformatics.ecogrid.ResultsetType;
import org.ecoinformatics.seek.ecogrid.exception.EcoGridException;
import java.util.Collections;
import ptolemy.vergil.tree.EntityTreeModel;
import ptolemy.vergil.tree.VisibleTreeModel;

/**
 * This class will handle quick search action
 */

public class QuickSearchAction extends AbstractAction
{
  // attributes
  private String          searchValue             = null;
  private Vector          searchScopeVector       = null;
  private CompositeEntity resultRoot              = null;
  private DebugPrinter    dbg                     = null;
  // This vector will store a list of vector. One sub-vector is a list of
  //resultrecord got from one search scope after excuting query (probably search
  // mutiple endpoints)
  private Vector          recordVector            = null;
  private DatasetPanel    datasetPanel            = null;
  private EntityTreeModel treeModel               = null;

  /**
   * Construct of this class
   * @param searchScopeVec the search info includes endspoints, namespace etc
   * @param resultRoot  the root to construct result panel
   * @param name  the name of the action
   * @param datasetPane  the datasetpane to fire the action
   */
  public QuickSearchAction(Vector searchScopeVect,
                           String name, DatasetPanel datasetPane)
  {
    super(name);
    dbg = DebugPrinter.getInstance();
    this.searchScopeVector = searchScopeVect;
    recordVector           = new Vector();
    this.datasetPanel      = datasetPane;

  }

  /**
   * Method to get the recordVector which store the ResultRecord after searching
   * @return Vector
   */
  public Vector getRecordVector()
  {
    return recordVector;
  }

  /**
   * Set up search scope vector
   * @param searchVector Vector
   */
  public void setSearchScope(Vector searchVector)
  {
    searchScopeVector = searchVector;
  }


  /**
   * The todo Implementation of abstract method. It will search ecogrid site
   *
   * @param e ActionEvent
   */
  public void actionPerformed(ActionEvent e)
  {
     if (datasetPanel != null)
     {
       searchValue = datasetPanel.getSearchTextFieldValue();
       resultRoot  = datasetPanel.getResultRoot();
     }
     if (searchScopeVector != null && !searchScopeVector.isEmpty() && resultRoot != null
         && searchValue != null && !searchValue.trim().equals(""))
     {
       resultRoot.removeAllEntities();
       // go through every namespace in search scope
       for (int i = 0; i < searchScopeVector.size(); i++)
       {
         // vecotr to store the ResultRecord for one search scope
         Vector resultFromOneScope = new Vector();
         SearchScope searchScope = (SearchScope)searchScopeVector.elementAt(i);
         //String namespace = searchScope.getNamespace();
         // get quick search query from metadata specification class
         MetadataSpecificationInterface metadataSpecClass =
                                      searchScope.getMetadataSpecification();
         QueryType quickSearchQuery = null;
         try
         {
           quickSearchQuery =
                    metadataSpecClass.getQuickSearchEcoGridQuery(searchValue);
         }
         catch(InvalidEcogridQueryException inE)
         {
           dbg.print("The error to generate quick search query " +
                     inE.getMessage(), 2);
           return;
         }
         Vector searchEndPoints = searchScope.getEndPoints();
         if ( searchEndPoints == null )
         {
           dbg.print("No search end points can be found", 2);
           return;
         }
         // go through the end points vector
         searchEndPointsVector(resultFromOneScope, searchEndPoints,
                               quickSearchQuery, metadataSpecClass);
         // sort the resultvector from one scope and add it to the recordVector
         //Collections.sort(resultFromOneScope, new ResultRecordComparator());
         recordVector.add(resultFromOneScope);

       }//for
       treeModel = new VisibleTreeModel(resultRoot);
       datasetPanel.update(treeModel);
     }//if
  }//actionPerformed

  /*cd
   * Go through the search end vector
   */
  private void searchEndPointsVector(Vector resultFromOneScope,
                             Vector searchEndPoints, QueryType quickSearchQuery,
                             MetadataSpecificationInterface metadataSpecClass)
  {

    // go through each search end points and get result set
    for (int j = 0; j < searchEndPoints.size(); j++)
    {

      String url = (String) searchEndPoints.elementAt(j);
      ResultsetType result = null;
      try
      {
        result = searchEndPoint(url, quickSearchQuery);
      }
      catch (EcoGridException ee)
      {
        dbg.print("Error in get result is " + ee.getMessage(), 2);
        continue;
      }
      // parse the resultset into ResultRecord array and stored it into
      // a vector
      ResultRecord[] recordArray = null;
      try
      {


        recordArray = metadataSpecClass.transformResultset(
                      result, url, resultRoot);

      }
      catch(Exception ee)
      {
        dbg.print("The error to transform from resultset to ResultRecord "+
                    ee.getMessage(), 2);
        continue;
      }
      // put the array into vector which will store the ResultRecord from one
      // search scope
      Util.transformResultRecordArrayToVector(recordArray, resultFromOneScope);
    }//for

  }

  /*
   * Search given end point by a given query
   */
  private ResultsetType searchEndPoint(String url, QueryType query)
                                       throws EcoGridException
  {
     ResultsetType results = null;
     String error          = null;
     QueryAction doQuery = new QueryAction(query, url);
     ActionEvent e = null;
     doQuery.actionPerformed(e);
     results = doQuery.getResultSet();
     error   = doQuery.getError();
     if ( results == null && error != null)
     {
       throw new EcoGridException(error);
     }
     return results;
  }

}
