/**
 *    '$RCSfile: SearchScopeController.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/05/19 21:42:17 $'
 *   '$Revision: 1.1 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.ecoinformatics.seek.ecogrid;

import edu.ucsb.nceas.configxml.ConfigXML;

import java.util.Vector;

/**
 * This class will get searchscope info (search end points, document namespace)
 * from registry and configure file.
 * @author Jing Tao
 *
 */

public class SearchScopeController
{
  private Vector searchScopeVector = new Vector();

  /**
   * Default Constructor to init searchscope
   */
  public SearchScopeController()
  {
    initSearchScope();
  }

  /**
   * Method to get search scope vector
   * @return Vector
   */
  public Vector getSearchScopeVector()
  {
    return searchScopeVector;
  }

   /*
    * Method get search scope from config file(The file path is hard coded)
    */
   private void initSearchScope()
   {

     try
     {
       ConfigXML config = new ConfigXML("lib/ecogridConfigure.xml");
       String namespacePath = "//"+MetadataSpecificationInterface.NAMESPACE;
       Vector namespaceVector = null;
       namespaceVector = config.getValuesForPath(namespacePath);

       if (namespaceVector != null)
       {
         SearchScope searchScope = null;
         for (int i=0; i<namespaceVector.size(); i++)
         {
           String namespace = (String)namespaceVector.elementAt(i);
           String endPointPath  = "//"+MetadataSpecificationInterface.ENDPOINT+
               "[@" + MetadataSpecificationInterface.NAMESPACE + "='" +
               namespace + "']";
           Vector endPoints = config.getValuesForPath(endPointPath);
           searchScope = new SearchScope(namespace, config, endPoints);
         }//for
         if (searchScope != null)
         {
            searchScopeVector.add(searchScope);
         }//if

       }//if
     }//try
     catch (Exception e)
     {
       System.err.println("Could NOT setup search scope from configure file");
     }//catch


   }


}
