/* A DigramTransitions actor...

@Copyright (c) 2002-2003 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION 2
                                                COPYRIGHTENDKEY
*/

package org.geon.test;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.DoubleToken;
import ptolemy.data.IntToken;
import ptolemy.data.LongToken;
import ptolemy.data.ObjectToken;
import ptolemy.data.RecordToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.RecordType;
import ptolemy.data.type.Type;
import ptolemy.data.expr.FileParameter;
import ptolemy.kernel.ComponentEntity;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;

import java.io.File;
import java.util.ArrayList;
import java.util.Map;
import javax.swing.JOptionPane;

//////////////////////////////////////////////////////////////////////////
//// DiagramTransitions
/**
This is a domain specific actor that holds all the information about for rock
naming diagrams and the transtions between them. It transfers the data along
with initial information to a process that loops over the rock data.

@author Efrat Jaeger
@version $Id: DiagramTransitions1.java,v 1.6 2004/07/01 23:32:50 higgins Exp $
@since Ptolemy II 3.0.2
*/
public class DiagramTransitions1 extends TypedAtomicActor
{

  /** Construct an actor with the given container and name.
 *  @param container The container.
 *  @param name The name of this actor.
 *  @exception IllegalActionException If the actor cannot be contained
 *   by the proposed container.
 *  @exception NameDuplicationException If the container already has an
 *   actor with this name.
 */
  public DiagramTransitions1(CompositeEntity container, String name)
    throws NameDuplicationException, IllegalActionException
  {
    super(container, name);
    diagramsAndTransitions = new TypedIOPort(this, "diagramsAndTransitions", false, true);
    diagramsAndTransitions.setTypeEquals(BaseType.OBJECT);
    index = new TypedIOPort(this, "index", false, true);
    index.setTypeEquals(BaseType.INT);
    minAbbrev = new FileParameter(this, "Minerals abbreviations");
    transitions = new FileParameter(this, "Transitions table");
    diagramsMapping = new FileParameter(this, "Diagrams mapping");

    _attachText("_iconDescription", "<svg>\n"
                + "<rect x=\"-25\" y=\"-20\" "
                + "width=\"50\" height=\"40\" "
                + "style=\"fill:yellow\"/>\n"
                + "<polygon points=\"-15,-2 0,-15 15,-2 11,15 -11,15\" "
                + "style=\"fill:white\"/>\n"
            + "</svg>\n");
  }

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

  /** All the diagrams and transitions information.
   */
  public TypedIOPort diagramsAndTransitions;

  /** A reference to the initial diagram.
   */
  public TypedIOPort index;

  /** Minerals abbreviations file.
   */
  public FileParameter minAbbrev;

  /** Transitions table file.
   */
  public FileParameter transitions;

  /** Mapping between diagrams and ids.
   */
  public FileParameter diagramsMapping;

  ///////////////////////////////////////////////////////////////////
  ////                         public methods                    ////

  /** If the specified attribute is <i>fileOrURL</i> and there is an
   *  open file being read, then close that file and open the new one;
   *  if the attribute is <i>numberOfLinesToSkip</i> and its value is
   *  negative, then throw an exception.  In the case of <i>fileOrURL</i>,
   *  do nothing if the file name is the same as the previous value of
   *  this attribute.
   *  @param attribute The attribute that has changed.
   *  @exception IllegalActionException If the specified attribute
   *   is <i>fileOrURL</i> and the file cannot be opened, or the previously
   *   opened file cannot be closed; or if the attribute is
   *   <i>numberOfLinesToSkip</i> and its value is negative.
   */
  public void attributeChanged(Attribute attribute)
          throws IllegalActionException {
        if (attribute == minAbbrev) {
            // NOTE: We do not want to close the file if the file
            // has not in fact changed.  We check this by just comparing
            // name, which is not perfect...
            if (_previousMinAbbrev != null
                    && !minAbbrev.getExpression().equals(_previousMinAbbrev)) {
                _previousMinAbbrev = minAbbrev.getExpression();
                minAbbrev.close();
                // Ignore if the fileOrUL is blank.
            }
        } else if (attribute == transitions) {
            if (_previousTransFile != null
                    && !transitions.getExpression().equals(_previousTransFile)) {
                _previousTransFile = transitions.getExpression();
            }
        } else if (attribute == diagramsMapping) {
            if (_prevDiagramsMapping != null
                    && !diagramsMapping.getExpression().equals(_prevDiagramsMapping)) {
            	_prevDiagramsMapping = diagramsMapping.getExpression();
            }
          } else {
          super.attributeChanged(attribute);
      }
  }

  /** Provide the diagrams and transitions between them along with a refernce to
   *  first diagram.
   *  @exception IllegalActionException If there's no director.
   */
  public void fire() throws IllegalActionException {

        String fileName = minAbbrev.getExpression();
        if (fileName == "" || fileName == null) {
          throw new IllegalActionException(this, "No abbreviation file was entered.");
        }
        File abbrevFile = minAbbrev.asFile();
        RockSample1 RS = new RockSample1();
        RS.createAbbrevTable(abbrevFile);
        File transFile = transitions.asFile();
        String transPath = transFile.getAbsolutePath();
        System.out.println(transPath);
        System.out.println("sth....");
        RS.setTransFile(transPath);
        File diagramId = diagramsMapping.asFile();
        RS.createDiagramIdMap(diagramId);

        ComponentEntity container = (ComponentEntity)this.getContainer();
        // reach the top level container and set the diagramsPath parameter.
        while (container.getContainer() != null) {
        	container = (ComponentEntity)container.getContainer();
        }
        StringAttribute diagramsPathParam = (StringAttribute) container.getAttribute("diagramsPath");

        diagramsPathParam.setExpression(RS.getDiagramsPath());

/*        Map tmp = RS.getAbbrevTable();
        System.out.println("hashtable size =" + tmp.size());
        ArrayList ar = (ArrayList) tmp.get(new String("A"));
        System.out.println("array size = " + ar.size());
        for (int i=0; i<ar.size();i++) {
        	System.out.println(ar.get(i));
        }*/
// RS.createTransitionTable(transFile); // TODO!!!
        //FIX ME: need to implement the transitions table as soon as we have more diagrams digitized.
        diagramsAndTransitions.broadcast(new ObjectToken(RS));
        index.broadcast(new IntToken(1));
  }

  /** Return false to indicate that the process has finished.
   *  @exception IllegalActionException If thrown by the super class.
   */
  public boolean postfire() throws IllegalActionException {
    return false;
  }

  ///////////////////////////////////////////////////////////////////
  ////                         private members                   ////

  /** Previous value of fileOrURL parameter. */
  private String _previousMinAbbrev;

  /** Previous value of fileOrURL parameter. */
  private String _previousTransFile;

  /** Previous value of fileOrURL parameter. */
  private String _prevDiagramsMapping;

}
